@@ -45,7 +45,7 @@ build_geos(){
     fi
     # Avoid building tests, depends on version
     case ${GEOS_VERSION} in
-        3.5.*|3.6.*|3.7.*)
+        3.7.*)
             BUILD_TESTING="";;
         3.8.*)
             BUILD_TESTING="-DBUILD_TESTING=ON";;
@@ -53,9 +53,10 @@ build_geos(){
             BUILD_TESTING="-DBUILD_TESTING=OFF";;
     esac
     cmake \
-        -DCMAKE_BUILD_TYPE=Release \
-        -DCMAKE_INSTALL_PREFIX=${GEOS_INSTALL} \
-        -DCMAKE_INSTALL_LIBDIR=lib \
+        -D CMAKE_BUILD_TYPE=Release \
+        -D CMAKE_INSTALL_PREFIX=${GEOS_INSTALL} \
+        -D CMAKE_INSTALL_LIBDIR=lib \
+        -D CMAKE_INSTALL_NAME_DIR=${GEOS_INSTALL}/lib \
         ${BUILD_TESTING} \
         ..
     cmake --build . -j 4
